import fastapi
from fastapi.security import OAuth2PasswordRequestForm

from schemas import CreateUserSchema, CreatedUserResponseSchema, RefreshTokenSchema
from ctrl import UserController
import services


user_router = fastapi.APIRouter(prefix='/users')
auth_router = fastapi.APIRouter(prefix='/auth')
ctrl = UserController()


@user_router.post('')
async def create_users(
        body: CreateUserSchema
) -> CreatedUserResponseSchema:
    return await ctrl.create(**body.model_dump(exclude_none=True))


@auth_router.post('/login')
async def login(
    form_data: OAuth2PasswordRequestForm = fastapi.Depends(),
):
    return await services.auth_service.login(form_data)


@user_router.get('/me')
async def read_user_me(
    user: dict = fastapi.Depends(services.auth_service.get_current_user)
):
    return user


@auth_router.post('/refresh')
async def refresh_token(
    data: RefreshTokenSchema,
    user: dict = fastapi.Depends(services.auth_service.get_current_user)
):
    return services.token_service.refresh_token(data.refresh_token)
