from uuid import UUID

import fastapi

from .schemas import ChapterBaseSchema
from .ctrl import ChapterController


router = fastapi.APIRouter(prefix='/chapters', tags=['Chapter'])
ctrl = ChapterController()


@router.get("")
async def get_chapters(
    query: ChapterBaseSchema = fastapi.Depends(),
):
    return await ctrl.get_list(**query.model_dump(exclude_none=True))


@router.get('/{id}')
async def get_chapter(
    id: UUID,
):
    return await ctrl.get(id)


@router.post('')
async def create_chapter(
    body: ChapterBaseSchema,
):
    return await ctrl.create(**body.model_dump(exclude_none=True))


@router.patch('/{id}')
async def update_chapter(
    id: UUID, body: ChapterBaseSchema,
):
    return await ctrl.update(id, **body.model_dump(exclude_none=True))


@router.delete('/{id}')
async def delete_chapter(
    id: UUID,
):
    return await ctrl.delete(id)
