from uuid import UUID

import fastapi

from .schemas import MaterialBaseSchema, MaterialPostSchema, MaterialSetSchema
from .ctrl import MaterialController
from ..dependencies import get_current_user


router = fastapi.APIRouter(prefix='/materials', tags=['Material'])
ctrl = MaterialController()


@router.get("")
async def get_materials(
    query: MaterialBaseSchema = fastapi.Depends(),
):
    return await ctrl.get_list(**query.model_dump(exclude_none=True))


@router.get('/{id}')
async def get_material(
    id: UUID,
):
    return await ctrl.get(id)



@router.post('')
async def create_material(
    body: MaterialPostSchema,
):
    return await ctrl.create(**body.model_dump(exclude_none=True))


@router.patch('/{id}')
async def update_material(
    id: UUID, body: MaterialPostSchema,
):
    return await ctrl.update(id, **body.model_dump(exclude_none=True))


@router.delete('/{id}')
async def delete_material(
    id: UUID,
):
    return await ctrl.delete(id)


@router.post('view_material')
async def set_view_material(
    token: str,
    body: MaterialSetSchema
):
    token_is_valid = await get_current_user(token)
    if token_is_valid:
        return await ctrl.create(**body.model_dump(exclude_none=True))
