from uuid import UUID

from db.repositories import MaterialRepository, ChapterRepository, UserMaterialViewRepository
from .schemas import MaterialSchema
from exceptions import common as common_exc, http as http_exc


class MaterialService:
    material_repository = MaterialRepository()
    chapter_repository = ChapterRepository()
    user_view_repo = UserMaterialViewRepository()

    async def get_material_list(self, **kwargs):
        materials = await self.material_repository.get_list(**kwargs)
        return [MaterialSchema.model_validate(material) for material in materials]

    async def get_material(self, id: UUID):
        material = await self.material_repository.get(id)
        return MaterialSchema.model_validate(material)

    async def create_material(self, **kwargs):
        chapter_uuid = kwargs.get('chapter_id')
        if chapter_uuid is None:
            raise http_exc.HTTPBadRequestException(detail="Validation error: Chapter must not be None")
        try:
            chapter = await self.chapter_repository.get(chapter_uuid)
        except common_exc.NotFoundException:
            raise http_exc.HTTPBadRequestException(detail="Chapter does not exist")

        material = await self.material_repository.create(
            name=kwargs.get('name'),
            text=kwargs.get('text'),
            chapter=chapter,
        )
        return MaterialSchema.model_validate(material)

    async def update_material(self, id: UUID, **kwargs):
        chapter_uuid = kwargs.get('chapter_id')
        if chapter_uuid:
            try:
                kwargs['chapter'] = await self.chapter_repository.get(chapter_uuid)
            except common_exc.NotFoundException:
                raise http_exc.HTTPBadRequestException(detail='Chapter does not exist')
        material = await self.material_repository.update(id, **kwargs)
        return MaterialSchema.model_validate(material)

    async def delete_material(self, id: UUID):
        return await self.material_repository.delete(id)

    async def set_material_view(self, user, **kwargs):
        material_uuid = kwargs.get('material_id')
        print(kwargs)
        if material_uuid is None:
            raise http_exc.HTTPBadRequestException(detail="Validation error: Material must not be None")
        try:
            material = await self.material_repository.get(material_uuid)
        except common_exc.NotFoundException:
            raise http_exc.HTTPBadRequestException(detail="Material does not exist")

        view = self.user_view_repo.create(user, material)
        return {'detail': 'success'}
